#include <avr/io.h>
#include "timer8.h"
#include <avr/interrupt.h>

#define TIMER_0_START_ADDRESS (0x0044)
#define TIMER_2_START_ADDRESS (0x00B0)

typedef volatile struct{
    unsigned char tccra;
    unsigned char tccrb;
    unsigned char ocra;
    unsigned char ocrb;
}timer_hw;

struct timer{
    unsigned char data;
    timer_hw volatile *p_hw;
};

static timer g_timer[2];
uint8_t temp_clock_type = 0x00;

void timer_init(void){
    g_timer[0].p_hw = (timer_hw volatile *)TIMER_0_START_ADDRESS;
    g_timer[1].p_hw = (timer_hw volatile *)TIMER_2_START_ADDRESS;
}

timer *timer_open(timer_type type){
    timer *p_timer = &(g_timer[type]);
    return p_timer;
}

void timer_set(timer *timer_temp, timer_mode mode, timer_clock clock, uint8_t duration_ms){
    timer_hw volatile *p_hw = timer_temp->p_hw;
    temp_clock_type = clock;
    p_hw->tccra |= mode;
    p_hw->ocra = duration_ms;
    TIMSK0=(0<<OCIE0B) | (1<<OCIE0A) | (0<<TOIE0);
}

void timer_start(timer *timer_temp){
    timer_hw volatile *p_hw = timer_temp->p_hw;
    p_hw->tccrb |= temp_clock_type;
}

void timer_stop(timer *timer_temp){
    timer_hw volatile *p_hw = timer_temp->p_hw;
    p_hw->tccrb = 0x00;
}

void timer_restart(timer *timer_temp){
    timer_hw volatile *p_hw = timer_temp->p_hw;
    p_hw->ocra = 0x00;
    p_hw->ocrb = 0x00;
}

void timer_clear(timer *timer_temp){
    timer_hw volatile *p_hw = timer_temp->p_hw;
    p_hw->tccra = 0x00;
    p_hw->tccrb = 0x00;
    p_hw->ocra = 0x00;
    p_hw->ocrb = 0x00;
}


ISR(TIMER0_OVF_vect){
    timer_callback();
}
